"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _findKey = _interopRequireDefault(require("@dword-design/functions/dist/find-key"));

var _keys = _interopRequireDefault(require("@dword-design/functions/dist/keys"));

var _replace = _interopRequireDefault(require("@dword-design/functions/dist/replace"));

var _some = _interopRequireDefault(require("@dword-design/functions/dist/some"));

var _startsWith = _interopRequireDefault(require("@dword-design/functions/dist/starts-with"));

var _babelPluginModuleResolver = require("babel-plugin-module-resolver");

var _path = _interopRequireDefault(require("path"));

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const isParentImport = path => /^(\.\/)?\.\.\//.test(path);

var _default = {
  meta: {
    type: 'suggestion',
    fixable: true,
    schema: [{
      type: 'object',
      properties: {
        aliases: {
          type: 'object',
          default: {
            '@': '.'
          }
        }
      },
      required: ['aliases']
    }]
  },
  create: context => {
    var _context$options$;

    const aliases = ((_context$options$ = context.options[0]) === null || _context$options$ === void 0 ? void 0 : _context$options$.aliases) || {
      '@': '.'
    };
    const path = context.getFilename(); // can't check a non-file

    if (path === '<text>') return {};
    return {
      ImportDeclaration: node => {
        var _ref, _aliases, _importPath2, _resolvedImport;

        const importPath = node.source.value;
        const hasAlias = (_ref = (_aliases = aliases, (0, _keys.default)(_aliases)), (0, _some.default)(alias => {
          var _importPath;

          return _importPath = importPath, (0, _startsWith.default)(alias)(_importPath);
        })(_ref));
        const resolvedImport = (0, _babelPluginModuleResolver.resolvePath)(importPath, path, {
          alias: aliases
        }); // relative parent

        if (_importPath2 = importPath, isParentImport(_importPath2)) {
          var _aliases2, _P$relative;

          const folder = _path.default.basename(path);

          const resolvedImportPath = _path.default.resolve(folder, importPath);

          const matchingAlias = (_aliases2 = aliases, (0, _findKey.default)(aliasPath => {
            var _resolvedImportPath;

            return _resolvedImportPath = resolvedImportPath, (0, _startsWith.default)(_path.default.resolve(aliasPath))(_resolvedImportPath);
          })(_aliases2));

          if (!matchingAlias) {
            return context.report({
              node,
              message: `Unexpected parent import '${importPath}'. No matching alias found to fix the issue`
            });
          }

          const rewrittenImport = `${matchingAlias}/${(_P$relative = _path.default.relative(_path.default.resolve(aliases[matchingAlias]), resolvedImportPath), (0, _replace.default)(/\\/g, '/')(_P$relative))}`;
          return context.report({
            node,
            message: `Unexpected parent import '${importPath}'. Use '${rewrittenImport}' instead`,
            fix: fixer => fixer.replaceTextRange([node.source.range[0] + 1, node.source.range[1] - 1], rewrittenImport)
          });
        }

        if (!(_resolvedImport = resolvedImport, isParentImport(_resolvedImport)) && hasAlias) {
          return context.report({
            node,
            message: `Unexpected subpath import via alias '${importPath}'. Use '${resolvedImport}' instead`,
            fix: fixer => fixer.replaceTextRange([node.source.range[0] + 1, node.source.range[1] - 1], resolvedImport)
          });
        }

        return undefined;
      }
    };
  }
};
exports.default = _default;
module.exports = exports.default;